/*-
 * #%L
 * Smile CDR - CDR
 * %%
 * Copyright (C) 2016 - 2025 Smile CDR, Inc.
 * %%
 * All rights reserved.
 * #L%
 */
package com.example.server.dstu2;

import ca.uhn.fhir.context.FhirContext;
import ca.uhn.fhir.rest.api.EncodingEnum;
import ca.uhn.fhir.rest.server.RestfulServer;
import ca.uhn.fhir.rest.server.interceptor.LoggingInterceptor;
import ca.uhn.fhir.rest.server.interceptor.ResponseHighlighterInterceptor;
import org.eclipse.jetty.ee10.servlet.ServletContextHandler;
import org.eclipse.jetty.ee10.servlet.ServletHolder;
import org.eclipse.jetty.server.Server;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.context.annotation.AnnotationConfigApplicationContext;

import java.util.List;

/**
 * This class is not used when deploying to Smile CDR as a Hybrid Provider class,
 * but it can be useful in order to test your data logic outside of Smile CDR in order
 * to speed up development cycles.
 */
public class Tester {

	private static final Logger ourLog = LoggerFactory.getLogger(Tester.class);

	public static void main(String[] theArgs) throws Exception {

		// Configuration
		Class<?> configClass = TestServerDstu2AppCtx.class;
		int port = 8080;
		FhirContext fhirContext = FhirContext.forDstu2();

		AnnotationConfigApplicationContext appCtx = new AnnotationConfigApplicationContext(configClass);
		List<?> resourceProviders = appCtx.getBean("resourceProviders", List.class);
		List<?> plainProviders = appCtx.getBean("plainProviders", List.class);

		Server server = new Server(port);

		RestfulServer hapiFhirRestfulServer = new RestfulServer(fhirContext);
		hapiFhirRestfulServer.registerProviders(resourceProviders);
		hapiFhirRestfulServer.registerProviders(plainProviders);
		hapiFhirRestfulServer.registerInterceptor(new ResponseHighlighterInterceptor());
		hapiFhirRestfulServer.registerInterceptor(new LoggingInterceptor());
		hapiFhirRestfulServer.setDefaultResponseEncoding(EncodingEnum.JSON);

		ServletContextHandler proxyHandler = new ServletContextHandler();
		proxyHandler.setContextPath("/");
		ServletHolder servletHolder = new ServletHolder();
		servletHolder.setServlet(hapiFhirRestfulServer);
		proxyHandler.addServlet(servletHolder, "/*");
		server.setHandler(proxyHandler);

		server.start();
		ourLog.info("FHIR Server listening at http://localhost:{}", port);
	}
}
